#include <project.h>
#include <stdio.h>

#define CAN_RX_MAILBOX_0_SHIFT      (1u)
#define CAN_RX_MAILBOX_1_SHIFT      (2u)
#define CAN_RX_MAILBOX_2_SHIFT      (4u)
#define DATA_SIZE                   (6u)
#define ONE_BYTE_OFFSET             (8u)

/* Function prototypes */
CY_ISR_PROTO(ISR_CAN);

volatile uint8 receiveMailboxNumber = 0xFFu;

int main()
{
    char8 txData[DATA_SIZE];

    uint16 Speed=0u , EngineRPM=0u;
    uint16 TravelDis1=0u , TravelDis2=0u;
    uint32 TravelDis=0u;
    
    UART_1_Start();
    CAN_Start();

    /* Set CAN interrupt handler to local routine */
    CyIntSetVector(CAN_ISR_NUMBER, ISR_CAN);    

    CyGlobalIntEnable;

    for(;;)
    {
        if (receiveMailboxNumber == CAN_RX_MAILBOX_Speed)
        {
            Speed=0u;
            Speed = ((uint16) ((uint16) CAN_RX_DATA_BYTE7(CAN_RX_MAILBOX_Speed) << ONE_BYTE_OFFSET)) | 
            CAN_RX_DATA_BYTE8(CAN_RX_MAILBOX_Speed);

            sprintf(txData, "Speed=%3d",Speed);
            UART_1_PutString(txData);
            UART_1_PutChar('\n');

            receiveMailboxNumber = 0xFFu;
        }
        
        if (receiveMailboxNumber == CAN_RX_MAILBOX_TravelDis)
        {
            TravelDis=0u;
            TravelDis1 = ((uint16) ((uint16) CAN_RX_DATA_BYTE1(CAN_RX_MAILBOX_TravelDis) << ONE_BYTE_OFFSET)) | 
            CAN_RX_DATA_BYTE2(CAN_RX_MAILBOX_TravelDis);
            
            TravelDis2 = ((uint16) ((uint16) CAN_RX_DATA_BYTE3(CAN_RX_MAILBOX_TravelDis) << ONE_BYTE_OFFSET)) | 
            CAN_RX_DATA_BYTE4(CAN_RX_MAILBOX_TravelDis);
            
            TravelDis=(TravelDis1<<16)|TravelDis2 ;
            
            sprintf(txData, "TravelDis=%3lu",TravelDis);
            UART_1_PutString(txData);
            UART_1_PutChar('\n');
            
            receiveMailboxNumber = 0xFFu;
        }
        
        if (receiveMailboxNumber == CAN_RX_MAILBOX_EngineRPM)
        {
            EngineRPM=0u;
            EngineRPM = ((uint16) ((uint16) CAN_RX_DATA_BYTE4(CAN_RX_MAILBOX_EngineRPM) << ONE_BYTE_OFFSET)) | 
            CAN_RX_DATA_BYTE5(CAN_RX_MAILBOX_EngineRPM);
            
            sprintf(txData, "EngineRPM=%3d",EngineRPM);
            UART_1_PutString(txData);
            UART_1_PutChar('\n');
            
            receiveMailboxNumber = 0xFFu;
        }
    }
}

CY_ISR(ISR_CAN)
{
    /* Clear Receive Message flag */
    CAN_INT_SR_REG.byte[1u] = CAN_RX_MESSAGE_MASK;

    /* Switch Status message received */
    if ((CY_GET_REG16((reg16 *) &CAN_BUF_SR_REG.byte[0u]) & CAN_RX_MAILBOX_0_SHIFT) != 0u)
    {        
        receiveMailboxNumber = CAN_RX_MAILBOX_EngineRPM;

        /* Acknowledges receipt of new message */
        CAN_RX_ACK_MESSAGE(CAN_RX_MAILBOX_EngineRPM);
    }

    /* ADC data message received */
    if ((CY_GET_REG16((reg16 *) &CAN_BUF_SR_REG.byte[0u]) & CAN_RX_MAILBOX_1_SHIFT) != 0u)
    {
        receiveMailboxNumber = CAN_RX_MAILBOX_TravelDis;

        /* Acknowledges receipt of new message */
        CAN_RX_ACK_MESSAGE(CAN_RX_MAILBOX_TravelDis);
    }
    
    if ((CY_GET_REG16((reg16 *) &CAN_BUF_SR_REG.byte[0u]) & CAN_RX_MAILBOX_2_SHIFT) != 0u)
    {
        receiveMailboxNumber = CAN_RX_MAILBOX_Speed;

        /* Acknowledges receipt of new message */
        CAN_RX_ACK_MESSAGE(CAN_RX_MAILBOX_Speed);
    }
    
}